/*
 *  prot_buddy.c
 *
 *  QQ Protocol. Part Buddy
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-7-12 Created.
 *
 *  Description: This file mainly includes the functions about 
 *
 */

#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "qqclient.h"
#include "memory.h"
#include "debug.h"
#include "qqpacket.h"
#include "packetmgr.h"
#include "protocol.h"
#include "buddy.h"

void prot_buddy_update_list( struct qqclient* qq, ushort pos )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_GET_BUDDY_LIST );
	bytebuffer *buf = p->buf;
	put_word( buf, pos );
	put_byte( buf, 0 );	//order??
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_update_list_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	ushort next_pos = get_word( buf );
	uint number;
	qqbuddy* b;
	while( buf->pos < buf->len ){
		number = get_int( buf );
		b = buddy_get( qq, number, 1 );
		b->face = get_word( buf );
		b->age = get_byte( buf );
		b->sex = get_byte( buf );
		uchar name_len = get_byte( buf );	//name_len
		get_data( buf,  (uchar*)b->nickname, name_len );	
		b->nickname[name_len] = 0;
		get_byte( buf );	//00 maybe
		get_word( buf );	//88 02
		get_byte( buf );	//40
		get_byte( buf );	//07
		get_byte( buf );	//C7
		get_byte( buf );	//0B
		get_byte( buf );	//01
	}
	if( next_pos != 0xffff ){
		prot_buddy_update_list( qq, next_pos );
	}else{
		DBG("buddy_count: %d", qq->buddy_list.count );
		buddy_set_all_off( qq );
		prot_buddy_update_signiture( qq, 0 );
		prot_buddy_update_account( qq, 0 );
		prot_buddy_update_alias( qq );
		buddy_put_event( qq );
	}
}


void prot_buddy_update_online( struct qqclient* qq, uint next_number )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_GET_BUDDY_ONLINE );
	bytebuffer *buf = p->buf;
	put_byte( buf, 0x02 );	//get buddies
	put_int( buf, next_number );
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_update_online_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uchar next_order = get_byte( buf );
	uint number;
	qqbuddy* b;
	uint max_number = 0;
	while( buf->pos < buf->len ){
		number = get_int( buf );
		if( number > max_number )
			max_number = number;
		b = buddy_get( qq, number, 0 );
		if( !b ){
			DBG("b == NULL");
			break;
		}
		get_byte( buf );
		b->ip = get_int( buf );
		b->port = get_word( buf );
		get_byte( buf );
		b->status = get_byte( buf );
		b->version = get_word( buf );
		get_data( buf, b->session_key, 16 );
		buf->pos += 11;
	}
	if( next_order != 0xff ){
		prot_buddy_update_online( qq, max_number );
	}else{
		buddy_put_event( qq );
		return;
	}
}


void prot_buddy_status( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uint number = get_int( buf );
	qqbuddy* b = buddy_get( qq, number, 0 );
	if( b ){
		get_byte( buf );
		b->ip = get_int( buf );
		b->port = get_word( buf );
		get_byte( buf );
		b->status = get_byte( buf );
		b->version = get_word( buf );
		get_data( buf, b->session_key, 16 );
		char event[64];
		sprintf( event, "buddystatus^$%u^$%s", number, buddy_status_string(b->status) );
		qqclient_put_event( qq, event );
	}
}


void prot_buddy_update_signiture( struct qqclient* qq, uint pos )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_GET_BUDDY_SIGN );
	bytebuffer *buf = p->buf;
	//sort
	buddy_sort_list( qq );
	put_byte( buf, 0x03 );	//command
	put_byte( buf, 0x00 );
	//start at pos
	int i, j, count;
	pthread_mutex_lock( &qq->buddy_list.mutex );
	for( i=0; i<qq->buddy_list.count; i++ )
		if( ((qqbuddy*)qq->buddy_list.items[i])->number >= pos )
			break;
	count = MIN( 50, qq->buddy_list.count-i );
	if( count == 0 ){
		pthread_mutex_unlock( &qq->buddy_list.mutex );
		DBG("signiture finished.");
		packetmgr_del_packet( &qq->packetmgr, p );
		return;
	}
	put_byte( buf, count );
	for( j=i; j<i+count; j++ ){
		put_int( buf, ((qqbuddy*)qq->buddy_list.items[j])->number );	//
		put_int( buf, 0 );
	}
	pthread_mutex_unlock( &qq->buddy_list.mutex );
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_update_signiture_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uchar cmd = get_byte( buf );
	switch( cmd ){
	case 0x03:
		{
			uchar result = get_byte( buf );
			if( result!=0 ){
				DBG("reuslt = %d", result );
				return;
			}
			uint next_pos;
			next_pos = get_int( buf );
			while( buf->pos < buf->len ){
				uint number = get_int( buf );
				qqbuddy* b = buddy_get( qq, number, 0 );
				if( !b ){
					DBG("b==NULL");
					return;
				}
				b->sign_time = get_int( buf );
				uchar len = get_byte( buf );
				len = MIN( len, SIGNITURE_LEN-1 );
				get_data( buf,  (uchar*)b->signiture, len );
				b->signiture[len] = 0;
			//	DBG("sign: %s  %s", b->nickname, b->signiture );
			}
			if( next_pos != 0 && next_pos != 0xffffffff ){
				//send else
				prot_buddy_update_signiture( qq, next_pos );
			}
		}
		break;
	default:
		DBG("unknown cmd = %x", cmd );
	}
}


void prot_buddy_update_account( struct qqclient* qq, uint pos )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_ACCOUNT );
	bytebuffer *buf = p->buf;
	//sort
	buddy_sort_list( qq );
	put_byte( buf, 0x01 );	//command
	put_int( buf, qq->number );
	//start at pos
	int i, j, count;
	pthread_mutex_lock( &qq->buddy_list.mutex );
	for( i=0; i<qq->buddy_list.count; i++ )
		if( ((qqbuddy*)qq->buddy_list.items[i])->number >= pos )
			break;
	count = MIN( 50, qq->buddy_list.count-i );
	if( count == 0 ){
		pthread_mutex_unlock( &qq->buddy_list.mutex );
		DBG("account finished.");
		packetmgr_del_packet( &qq->packetmgr, p );
		return;
	}
	put_byte( buf, count );
	for( j=i; j<i+count; j++ ){
		put_int( buf, ((qqbuddy*)qq->buddy_list.items[j])->number );	//
	}
	pthread_mutex_unlock( &qq->buddy_list.mutex );
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_verify_msg( struct qqclient* qq, uchar cmd, uchar type, uint number, token *tok )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_ACCOUNT );
	bytebuffer *buf = p->buf;
	put_byte( buf, cmd );
	put_byte( buf, type );
	put_int( buf, number );
	switch( cmd ){
	case 3:	//
		put_byte( buf, 0 );
		put_byte( buf, tok->len );
		put_data( buf, tok->data, tok->len );
		break;
	}
	post_packet( qq, p, SESSION_KEY );
}


void prot_buddy_update_account_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uchar cmd = get_byte( buf );
	switch( cmd ){
	case 0x01:
		{
			get_int(buf );	//self number
			ushort result = get_word( buf );
			if( result!=0x0001 ){
				DBG("reuslt = %d", result );
				return;
			}
			uint next_pos;
			next_pos = get_int( buf );
			int count;
			count = get_word( buf );
			while( buf->pos < buf->len ){
				uint number = get_int( buf );
				qqbuddy* b = buddy_get( qq, number, 0 );
				if( !b ){
					DBG("b==NULL");
					return;
				}
				b->account_flag = get_int( buf );
				if( b->account_flag > 0 ){
					uchar len = get_byte( buf );
					len = MIN( len, ACCOUNT_LEN-1 );
					get_data( buf,  (uchar*)b->account, len );
					b->account[len] = 0;
				//	DBG("account: %s  %s", b->nickname, b->account );
				}
			}
			if( next_pos != 0 && next_pos != 0xffffffff ){
				//send else
				prot_buddy_update_account( qq, next_pos );
			}
		}
		break;
	case 0x03:
		{
			uchar type, result;
			uint number;
			type = get_byte( buf );
			number = get_int( buf );
			result = get_byte( buf );
			if( result != 00 ){
				DBG("failed to verify adding buddy msg.");
			}
			DBG("verified adding buddy msg %u result: %d", number, result );
		}
		break;
	default:
		DBG("unknown cmd = %x", cmd );
	}
}


void prot_buddy_update_alias( struct qqclient* qq )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_BUDDY_ALIAS );
	bytebuffer *buf = p->buf;
	put_byte( buf, 0x04 );	//command
	put_byte( buf, 0x00 );
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_update_alias_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uchar cmd = get_byte( buf );
	switch( cmd ){
	case 0x04:
		{
			uchar result = get_byte( buf );
			if( result!=0x01 ){
				DBG("reuslt = %d", result );
				return;
			}
			while( buf->pos < buf->len ){
				uint number = get_int( buf );
				qqbuddy* b = buddy_get( qq, number, 0 );
				if( !b ){
					DBG("b==NULL");
					return;
				}
				uchar len = get_byte( buf );
				len = MIN( len, ALIAS_LEN-1 );
				get_data( buf,  (uchar*)b->alias, len );
				b->alias[len] = 0;
			//	DBG("alias: %s  %s", b->nickname, b->alias );
			}
			buddy_put_event( qq );
		}
		break;
	default:
		DBG("unknown cmd = %x", cmd );
	}
}


void prot_buddy_add_buddy( struct qqclient* qq, uchar cmd, uint number, token* tok )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_ADD_BUDDY );
	bytebuffer *buf = p->buf;
	put_byte( buf, cmd );	//command
	put_int( buf, number );
	switch( cmd ){
	case 03:
		put_byte( buf, 0x00 );
		put_byte( buf, 0x00 );
		put_byte( buf, 0x00 );
		break;
	case 01:
		put_word( buf, 0x0000 );
		put_byte( buf, tok->len );
		put_data( buf, tok->data, tok->len );
		put_byte( buf, 0x00 );
		break;
	}
	post_packet( qq, p, SESSION_KEY );
}

void prot_buddy_add_buddy_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uchar cmd = get_byte( buf );
	uint number;
	number = get_int( buf );
	switch( cmd ){
	case 0x03:
		{
			uchar result = get_byte( buf );
			if( result == 0x00 ){
				char msg[128];
				DBG("add buddy %u ok!!", number);
				sprintf( msg, "Ѿ[%u]Ϊĺѡ", number );
				buddy_msg_callback( qq, 10000, time(NULL), msg );
				//refresh buddylist
				buddy_update_list( qq );
			}else{
				DBG("failed to add buddy %u  result=%d", number, result );
			}
		}
		break;
	case 0x01:
		{
			uchar result = get_byte( buf );
			if( result == 0x00 ){
				char msg[128];
				DBG("add buddy %u ok!!", number);
				sprintf( msg, "Ѿ[%u]Ϊĺѡ", number );
				buddy_msg_callback( qq, 10000, time(NULL), msg );
				//refresh buddylist
				buddy_update_list( qq );
			}else{
				DBG("failed to add buddy %u  result=%d", number, result );
			}
		}
		break;
	default:
		DBG("unknown cmd = %x", cmd );
	}
}
