#include <time.h>
#include <stdlib.h>
#include <string.h>
#ifdef __WIN32__
#include <winsock.h>
#include <wininet.h>
#else
/*
 *  prot_im.c
 *
 *  QQ Protocol. Part Internet Message
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-7-12 Created.
 *
 *  Description: This file mainly includes the functions about 
 *  sending/receiving buddy or qun messages
 *
 */

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif

#include "qqclient.h"
#include "memory.h"
#include "debug.h"
#include "qqpacket.h"
#include "packetmgr.h"
#include "qqcrypt.h"
#include "md5.h"
#include "buddy.h"
#include "protocol.h"


void prot_im_send_msg( struct qqclient* qq, uint to, char* msg )
{
	int len = strlen( msg );
	if( len > 700 ){
		return ;
	}
//	DBG("str: %s  len: %d", msg, len );
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_SEND_IM );
	bytebuffer *buf = p->buf;
	put_int( buf, qq->number );
	put_int( buf, to );
	put_word( buf, qq->version );
	put_int( buf, qq->number );
	put_int( buf, to );
	put_data( buf, qq->im_key, 16 );
	put_word( buf, QQ_NORMAL_IM_TEXT );	//message type
	put_word( buf, p->seqno );
	put_int( buf, p->time_create );
	put_word( buf, 0 );
	put_int( buf, 1 );	//has font attribute
	put_byte( buf, 1 );	//slice_count
	put_byte( buf, 0 );	//slice_no
	put_byte( buf, rand() );	//msg_id??
	put_byte( buf, 0 );
//	put_word( buf, p->seqno );
	put_byte( buf, QQ_IM_TEXT );	//auto_reply
	put_data( buf, (uchar*)msg, len );
	put_byte( buf, ' ' );
	put_byte( buf, 0 );	//end message
	put_byte( buf, 0x0A );	//font attribute
	put_byte( buf, 0 );	//R
	put_byte( buf, 0 );	//G
	put_byte( buf, 0 );	//B
	put_byte( buf, 0 );	//?
	put_word( buf, 0x8602 );	//GB2312
	put_int( buf, 0xCBCECCE5 );	//
	put_byte( buf, 0x0D );	////len of font attr
	post_packet( qq, p, SESSION_KEY );
}

void prot_im_ack_recv( struct qqclient* qq, qqpacket* pre )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_RECV_IM );
	bytebuffer *buf = p->buf;
	p->seqno = pre->seqno;
	put_data( buf, pre->buf->data, 16 );
	p->need_ack = 0;
	post_packet( qq, p, SESSION_KEY );
}

static void process_buddy_im_text( struct qqclient* qq, qqpacket* p, qqmessage* msg )
{
	bytebuffer *buf = p->buf;
	get_word( buf );	//session id
	msg->msg_time = get_int( buf );
	get_word( buf );	//face
	get_byte( buf );
	get_byte( buf );
	get_byte( buf );
	get_byte( buf );	//font_attr
	//Ƭ
	msg->slice_count = get_byte( buf );
	msg->slice_no = get_byte( buf );
	msg->msg_id = get_word( buf );
	msg->auto_reply = get_byte( buf );
	buf->pos += 8;
	get_string( buf, msg->msg_content, MSG_CONTENT_LEN );
//	DBG("buddy msg from %u:", msg->from );
//	puts( msg->msg_content );
	buddy_msg_callback( qq, msg->from, msg->msg_time, msg->msg_content );
}

static void process_buddy_im( struct qqclient* qq, qqpacket* p, qqmessage* msg )
{
	bytebuffer *buf = p->buf;
	get_word( buf );	//version
	msg->from = get_int( buf );
	if( get_int( buf ) != qq->number ){
		DBG("nothing but this is impossible!!");
	}
	//to check if this buddy is in our list, or we add it.
	buddy_get( qq, msg->from, 1 );
	//IM key
	uchar key[16];
	get_data( buf, key, 16 );
	ushort content_type = get_word( buf );
	switch( content_type ){
	case QQ_NORMAL_IM_TEXT:
	//	DBG("QQ_NORMAL_IM_TEXT");
		process_buddy_im_text( qq, p, msg );
		break;
	case QQ_NORMAL_IM_FILE_REQUEST_TCP:
		DBG("QQ_NORMAL_IM_FILE_REQUEST_TCP");
		break;
	case QQ_NORMAL_IM_FILE_APPROVE_TCP:
		DBG("QQ_NORMAL_IM_FILE_APPROVE_TCP");
		break;
	case QQ_NORMAL_IM_FILE_REJECT_TCP:
		DBG("QQ_NORMAL_IM_FILE_REJECT_TCP");
		break;
	case QQ_NORMAL_IM_FILE_REQUEST_UDP:
		DBG("QQ_NORMAL_IM_FILE_REQUEST_UDP");
		break;
	case QQ_NORMAL_IM_FILE_APPROVE_UDP:
		DBG("QQ_NORMAL_IM_FILE_APPROVE_UDP");
		break;
	case QQ_NORMAL_IM_FILE_REJECT_UDP:
		DBG("QQ_NORMAL_IM_FILE_REJECT_UDP");
		break;
	case QQ_NORMAL_IM_FILE_NOTIFY:
		DBG("QQ_NORMAL_IM_FILE_NOTIFY");
		break;
	case QQ_NORMAL_IM_FILE_PASV:
		DBG("QQ_NORMAL_IM_FILE_PASV");
		break;
	case QQ_NORMAL_IM_FILE_CANCEL:
		DBG("QQ_NORMAL_IM_FILE_CANCEL");
		break;
	case QQ_NORMAL_IM_FILE_EX_REQUEST_UDP:
//		DBG("QQ_NORMAL_IM_FILE_EX_REQUEST_UDP");
		break;
	case QQ_NORMAL_IM_FILE_EX_REQUEST_ACCEPT:
		DBG("QQ_NORMAL_IM_FILE_EX_REQUEST_ACCEPT");
		break;
	case QQ_NORMAL_IM_FILE_EX_REQUEST_CANCEL:
		DBG("QQ_NORMAL_IM_FILE_EX_REQUEST_CANCEL");
		break;
	case QQ_NORMAL_IM_FILE_EX_NOTIFY_IP:
		DBG("QQ_NORMAL_IM_FILE_EX_NOTIFY_IP");
		break;
	default:
		DBG("UNKNOWN type: %x", content_type );
		break;
	}
}

static void process_sys_im( struct qqclient* qq, qqpacket* p, qqmessage* msg )
{
	bytebuffer *buf = p->buf;
	msg->msg_time = time(NULL);
	uchar content_type;
	content_type = get_byte( buf );
	uchar len = get_byte( buf );
	get_data( buf, (uchar*)msg->msg_content, len );
	msg->msg_content[len] = 0;
	buddy_msg_callback( qq, msg->from, msg->msg_time, msg->msg_content );
	if( strstr( msg->msg_content, "Σ" ) != NULL ){
		qqclient_set_process( qq, P_ERROR );
	}else{
		qqclient_set_process( qq, P_BUSY );
	}
//	DBG("sysim(type:%x): %s", content_type, msg->msg_content );
}

static void process_qun_im( struct qqclient* qq, qqpacket* p, qqmessage* msg )
{
	bytebuffer *buf = p->buf;
	get_int( buf );		//ext_number
	get_byte( buf );	//normal qun or temp qun?
	msg->from = get_int( buf );
	get_word( buf );	//zero
	msg->msg_id = get_word( buf );
	msg->msg_time = get_int( buf );
	buf->pos += 16;
	get_string( buf, msg->msg_content, MSG_CONTENT_LEN );
	
//	DBG("process_qun_im(number:%u): ", msg->from );
//	puts( msg->msg_content );
	if( msg->from == qq->number )
		return;
	qun_msg_callback( qq, msg->from, msg->qun_number, msg->msg_time, msg->msg_content );
}

static void process_qun_member_im( struct qqclient* qq, qqpacket* p, qqmessage* msg )
{
	bytebuffer *buf = p->buf;
	token tok_unknown;
	buf->pos += 14;	//00 65 01 02 00 00 00 00 00 00 00 00 00 00
	get_token( buf, &tok_unknown );	//len: 0x30
	buf->pos += 8;	//00 26 00 1c 02 00 01 00
	get_int( buf );	//unknown time 48 86 b9 e5
	get_int( buf );	//00 01 51 80
	msg->from = get_int( buf ); //from 10 4a 61 e3
	get_int( buf );	//self number
	get_int( buf );	//00 00 00 1f
	msg->msg_time = get_int( buf );	//48 86 c4 f1
	msg->qun_number = get_int( buf );	//1c aa 5d 98 
	get_int( buf );		//ext_number
	buf->pos += 15;	//00 0d 00 00 00 00 00 00 00 00 00 00 00 00 00 
	process_buddy_im( qq, p, msg );
	DBG("process_qun_member_im: qun_number: %u", msg->qun_number );
}


void prot_im_recv_msg( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	uint from, from_ip;
	ushort from_port;
	ushort im_type;
	qqmessage *msg;
	if( !qq->login_finish ){	//not finished login
		DBG("Early message ... Abandoned.");
		return;
	}
	NEW( msg, sizeof( qqmessage ) );
	//
	from = get_int( buf );
	if( get_int( buf ) != qq->number ){
		DBG("nothing is impossible!!");
	}
	get_int( buf );	//sequence
	from_ip = get_int( buf );
	from_port = get_word( buf );
	im_type = get_word( buf );
	switch( im_type ){
	case QQ_RECV_IM_TO_BUDDY:
		DBG("QQ_RECV_IM_TO_BUDDY");
		break;
	case QQ_RECV_IM_TO_UNKNOWN:
		DBG("QQ_RECV_IM_TO_UNKNOWN");
		break;
	case QQ_RECV_IM_NEWS:
		DBG("QQ_RECV_IM_NEWS");
		break;
	case QQ_RECV_IM_UNKNOWN_QUN_IM:
		DBG("QQ_RECV_IM_UNKNOWN_QUN_IM");
		break;
	case QQ_RECV_IM_ADD_TO_QUN:
		DBG("QQ_RECV_IM_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_DEL_FROM_QUN:
		DBG("QQ_RECV_IM_DEL_FROM_QUN");
		break;
	case QQ_RECV_IM_APPLY_ADD_TO_QUN:
		DBG("QQ_RECV_IM_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_APPROVE_APPLY_ADD_TO_QUN:
		DBG("QQ_RECV_IM_APPROVE_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_REJCT_APPLY_ADD_TO_QUN:
		DBG("QQ_RECV_IM_REJCT_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_CREATE_QUN:
		DBG("QQ_RECV_IM_CREATE_QUN");
		break;
	case QQ_RECV_IM_TEMP_QUN_IM:
		DBG("QQ_RECV_IM_TEMP_QUN_IM");
	case QQ_RECV_IM_QUN_IM:
		msg->msg_type = MT_QUN;
		msg->qun_number = from;
		process_qun_im( qq, p, msg );
		break;
	case QQ_RECV_IM_SYS_NOTIFICATION:
		msg->msg_type = MT_SYSTEM;
		process_sys_im( qq, p, msg );
		break;
	case QQ_RECV_IM_BUDDY:
		msg->msg_type = MT_BUDDY;
		process_buddy_im( qq, p, msg );
		break;
	case QQ_RECV_IM_WRITING:
		DBG("QQ_RECV_IM_WRITING");
		break;
	case QQ_RECV_IM_QUN_MEMBER_IM:
		msg->msg_type = MT_QUN_MEMBER;
		process_qun_member_im( qq, p, msg );
		break;
	default:
		DBG("Unknown message type : %x", im_type );
	}
	//ack recv
	prot_im_ack_recv( qq, p );
	DEL( msg );
}

