/*
 *  prot_misc.c
 *
 *  QQ Protocol. Part Miscellaneous
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-7-12 Created.
 *
 *  Description: This file mainly includes the functions about 
 *
 */

#include <time.h>
#include <stdlib.h>
#include <string.h>
#ifdef __WIN32__
#include <winsock.h>
#include <wininet.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif

#include "qqclient.h"
#include "memory.h"
#include "debug.h"
#include "qqpacket.h"
#include "packetmgr.h"
#include "qqcrypt.h"
#include "md5.h"
#include "protocol.h"


void prot_misc_get_ad( struct qqclient* qq, uchar cmd )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_GET_AD );
	bytebuffer *buf = p->buf;
	if( cmd == 0x07 ){
		int i;
		put_byte( buf, cmd );
		for( i=0; i<5; i++ )
			put_byte( buf, 0 );
	}else{
		DBG("not implemented cmd: %x", cmd );
	}
	post_packet( qq, p, SESSION_KEY );
}

void prot_misc_get_ad_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
//	hex_dump( buf->data, buf->len );
	uchar cmd = get_byte( buf );	//
	if( cmd == 0x07 ){
	//	hex_dump( buf->data, buf->len );
	}else{
		DBG("not implemented cmd: %x", cmd );
	}
}



void prot_misc_unknown_2e( struct qqclient* qq )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_UNKNOWN_2E );
	bytebuffer *buf = p->buf;
	p->need_ack = 0;
	put_word( buf, 0 );
	post_packet( qq, p, SESSION_KEY );
}

void prot_misc_unknown_2e_reply( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	ushort d = get_word( buf );	//
	if( d != 0x0001 ){
		hex_dump( buf->data, buf->len );
	}
}


void prot_misc_unknown_b8( struct qqclient* qq )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_UNKNOWN_B8 );
	bytebuffer *buf = p->buf;
	p->need_ack = 0;
	put_byte( buf, 1 );
	post_packet( qq, p, SESSION_KEY );
}

void prot_misc_unknown_b8_reply( struct qqclient* qq, qqpacket* p )
{
//	bytebuffer *buf = p->buf;
	//unknown
}

void prot_misc_unknown_c6( struct qqclient* qq )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_UNKNOWN_C6 );
	bytebuffer *buf = p->buf;
	p->need_ack = 0;
	put_byte( buf, 2 );
	put_int( buf, 0 );
	post_packet( qq, p, SESSION_KEY );
}

void prot_misc_unknown_c6_reply( struct qqclient* qq, qqpacket* p )
{
//	bytebuffer *buf = p->buf;
	//unknown
}

void prot_misc_version( struct qqclient* qq )
{
	qqpacket* p = packetmgr_new_send( qq, QQ_CMD_VERSION );
	bytebuffer *buf = p->buf;
	p->need_ack = 0;
	put_byte( buf, 6 );
	put_int( buf, 0 );
	put_byte( buf, 0 );
	post_packet( qq, p, SESSION_KEY );
}

void prot_misc_version_reply( struct qqclient* qq, qqpacket* p )
{
//	bytebuffer *buf = p->buf;
	//see my doc
}

void prot_misc_broadcast( struct qqclient* qq, qqpacket* p )
{
	bytebuffer *buf = p->buf;
	char  e[4][256];
	char* str = (char*)buf->data;
	int i, len, s, j;
	len = buf->len;
	memset( e, 0, sizeof(e) );
	for( i=0, s=0, j=0; i<=len && j<4; i++ ){
		if( str[i] == 0x1f || i==len ){
			memcpy( e[j], &str[s], MIN(i-s, 255) );
			j++;
			s = i+1;
		}
	}
	if( strcmp( e[0], "06" ) == 0 ){
		//broadcast
		char event[384];
		sprintf( event, "broadcast^$%s^$%s", e[1], e[3] );
		qqclient_put_event( qq, event );
	}else if( strcmp( e[0], "41" ) == 0 ){
		uint from, to;
		uchar type;
		token tok;
		from = atoi( e[1] );
		to = atoi( e[2] );
		uchar len = e[3][0];
		uchar* p = (uchar*)(&e[3][1]+len);
		type = *p++;
		if( type == 1 ){
			p++;
			tok.len = *p++;
			memcpy( tok.data, p, MIN(tok.len, TOKEN_LEN) );
			prot_buddy_verify_msg( qq, 03, type, from, &tok );
			prot_buddy_add_buddy( qq, 03, from, NULL );
		}
		//e[0] e[1] e[2] can be reused 
		if( len > 0 ){
			strncpy( e[0], &e[3][1], len );
			e[0][len] = 0;
		}else{
			strcpy( e[0], "Nothing" );
		}
		sprintf( e[1], "[%u]Ϊѡԣ%s", from, e[0] );
		buddy_msg_callback( qq, 10000, time(NULL), e[1] );
	}else if( strcmp( e[0], "04") == 0 ){
		char event[384];
		sprintf( event, "broadcast^$û[%s]ܾΪѡ", e[2] );
		qqclient_put_event( qq, event );
	}else if( strcmp( e[0], "40" ) == 0 ){
		uint from, to;
		token tok_verify, tok_add;
		from = atoi( e[1] );
		to = atoi( e[2] );
		uchar* p = (uchar*)e[3];
		tok_add.len = *p++;
		memcpy( tok_add.data, p, MIN(tok_add.len, TOKEN_LEN) );
		p+=tok_add.len;
		p++;	//00
		tok_verify.len = *p++;
		memcpy( tok_verify.data, p, MIN(tok_verify.len, TOKEN_LEN) );
		prot_buddy_verify_msg( qq, 03, 1, from, &tok_verify );
		prot_buddy_add_buddy( qq, 01, from, &tok_add );
		//e[0] e[1] e[2] can be reused 
		sprintf( e[1], "[%u]Ϊѣ", from );
		buddy_msg_callback( qq, 10000, time(NULL), e[1] );
	}else{
		DBG("e[0]: %s", e[0] );
	}
}
